function [anomalies,cropped_dates] = calcAnomalies(data,date_vector,varargin)

% Calculates anomalies relative to a user-specified trend
% and to the climatological seasonal cycle.
%
% USAGE: [anomalies,cropped_dates] = calcAnomalies(data,date_vector)
%        [anomalies,cropped_dates] = calcAnomalies(__,trend)
%        [anomalies,cropped_dates] = calcAnomalies(__,start_year,stop_year);
%        [anomalies,cropped_dates] = calcAnomalies(__,trend,start_year,stop_year);
%
%        data and anomalies are vectors. date_vector is a Matlab date vector.
%        cropped_dates is the cropped date vector.
%
% Optional parameter are
%
%        trend:      trend that is to be subtracted from the data
%                    prior to the removal of the seasonal cycle.
%                    Vector that has the same length as data.
%                    Default: When no trend is provided, the linear
%                    trend is calculated and subtracted from
%                    the data.
%
%        start_year: When should anomaly calculation begin?
%
%        stop_year:  When should it stop?
%
% IF YOU DON'T WHISH TO CALCULATE A TREND,
% specify a "dummy trend" that consists of zeros and
% has the same size as the data.

% ----------- %
% PARSE INPUT %
% ----------- %

    n_additional = length(varargin);

    % define defaults:
        trend_data = [];
        start_year = min(date_vector(:,1));
        stop_year  = max(date_vector(:,1));

    switch n_additional
        case 0
        case 1
            trend_data = varargin{1};
        case 2
            start_year = varargin{1};
            stop_year  = varargin{2};
        case 3
            trend_data = varargin{1};
            start_year = varargin{2};
            stop_year  = varargin{3};
        otherwise
            error('Wrong amount of input variables specified.');
    end

% -------------- %
% CONSTRAIN TIME %
% -------------- %

    is_time =  ( date_vector(:,1) >= start_year ) & ( date_vector(:,1) <= stop_year );

    % cropping
    date_vector = date_vector(is_time,:);
    data        = data(is_time);

    % need to crop the trend data as well if it is not empty
    if ~isempty(trend_data)
        trend_data = trend_data(is_time);
    end

    % for returning
    cropped_dates = date_vector;

    % use datenums as x-axis
    x_values = datenum(date_vector);

% ------------------ %
% DATA QUALITY CHECK %
% ------------------ %

    is_okay  = ~(isnan(data));

    % anything else but NaNs?
    if ( isempty(data(is_okay)) )
        anomalies = nan(size(data));
        return
    end

% ------------ %
% DETREND DATA %
% ------------ %

    if isempty(trend_data)

%          disp('calculating linear trend');

        % need to have NaN-free data for regression
        valid_data  = data(is_okay);
        valid_times = x_values(is_okay);

        % fitting: use only good values
        vals_poly = polyfit(valid_times,valid_data,1);

        % evaluation: use ALL values
        trend_data = polyval(vals_poly,x_values);

    % trend data compatible with data?
    else
        if ( length(data) ~= length(trend_data) )
            error('Data and trend data have different sizes.');
        end
    end

    % assign anomalies relative to the trend to
    % the correct piece of monthlyAnomalies
    detrended = data - trend_data;

% ------------------------------------ %
% REMOVE CLIMATOLOGICAL SEASONAL CYCLE %
% ------------------------------------ %

    anomalies = zeros(size(data));
    seasonal_cycle = zeros(1,12);

    for month = 1:12

        % timing
        is_month = ( date_vector(:,2) == month );


        % climatology
        climatology = nanmean(detrended(is_month));

        % assign anomalies
        anomalies(is_month)   = detrended(is_month)-climatology;
        seasonal_cycle(month) = nanmean(data(is_month));

    end
