%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                               %
%   COMPARE FEEDBACK STRENGTHS IN ATL3/NINO34   %
%   MAIN                                        %
%                                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all;
compare_aviso_oras4_strengths_Atl3_Nino34_settings;

disp(sprintf('\n%s: %s-%s',basin_name,upper(reference_name),upper(lag_name)));

% --------------------------------- %
% LOAD DATA AND CALCULATE ANOMALIES %
% --------------------------------- %

    anomalies = struct();

    for c_dataset = 1:n_datasets

        dataset_name = char(all_datasets{c_dataset});

        for var_name = {reference_name,lag_name}

            var_name    = char(var_name);
            var_dataset = all_data_names.(dataset_name).(var_name);
            index_name  = all_indices.(basin_name).(var_name);

            % infile name
            infile = strrep(infile_dummy,'@DATASET_NAME@',upper(var_dataset));
            infile = strrep(infile,'@VAR_NAME@',var_name);
            infile = strrep(infile,'@INDEX_NAME@',index_name);

            % load
            data = load(infile);

            % anomalies
            [current_anomalies cropped_dates] = ...
                    calcAnomalies(data.index,data.date_vector,start_year,stop_year);

            % check recursively that date vectors are consistent
            current_dv = cropped_dates;

             % set days, hours, minutes, seconds to 0
            current_dv(:,3:end) = 0;

            if isfield(anomalies,'date_vector')

                last_dv = anomalies.date_vector;

                % same size?
                same_size = ( sum(size(current_dv) == size(last_dv)) == 2 );

                if ~same_size
                    error('Cropped date vectors are not of the same size');
                end

                % same values
                n_steps = size(current_dv,1);

                is_equal = ( sum(datenum(current_dv) == datenum(last_dv)) == n_steps );

                if ~is_equal
                    error('Cropped date vectors are not equal. Check that all date vectors overlap.');
                end

            end

            % assign data
            anomalies.(dataset_name).(var_name) = current_anomalies;
            anomalies.date_vector = current_dv;

        % for: load variables
        end

    % for: load datasets
    end

if perform_calculations
% -------------------- %
% PERFORM CALCULATIONS %
% -------------------- %

    % switch off warning
    warning('off',warn_id);

    strengths     = struct();  % for both datasets
    significances = struct();

    textprogressbar('Calculating feedback strengths: ');
    c_step = 1;
    for c_dataset = 1:n_datasets

        % select data
        dataset_name = char(all_datasets{c_dataset});
        use_ref      = anomalies.(dataset_name).(reference_name);
        use_lag      = anomalies.(dataset_name).(lag_name);

        for month = 1:12

            % diagnose lags
            % --> cropped time series
            [all_strengths,lag_value,cropped_ref,cropped_lag] = diagnoseLag(...
                    use_ref,anomalies.date_vector,...
                    use_lag,anomalies.date_vector,...
                    analysis_type,...
                    month,maximum_lag,start_year,stop_year);


            % calculate positive and negative feedback strengths
            for anomaly_type = [-1 1]
            textprogressbar(c_step/(n_datasets*12*2)*100);


                % assign correct anomaly type
                switch anomaly_type
                    case -1
                        type_string = 'negative';
                    case 1
                        type_string = 'positive';
                end

                % select correct piece of data
                is_anomaly  = ( sign(cropped_ref) == anomaly_type );
                sample_size = length(find(is_anomaly));

                % ignore cases where the sample size does not allow even robust regression
                % i.e. when there's only two samples
                if sample_size < 3

                    current_strength = NaN;
                    is_significant   = 0;

                else

                    % calculate actual feedback strength
                    current_strength =  calculateFeedback(...
                            cropped_ref(is_anomaly),cropped_lag(is_anomaly),analysis_type);

                    % bootstrap additional strengths
                    bootstrapped_strengths = bootstrapFeedbackStrength(...
                                    cropped_ref,cropped_lag,...
                                    sample_size,n_bootstraps,analysis_type);

                    % perform significance test
                    is_significant = testValue(bootstrapped_strengths,current_strength,alpha);

                end

                % assign data
                strengths.(dataset_name).(type_string)(month)     = current_strength;
                significances.(dataset_name).(type_string)(month) = is_significant;

                % counting
                c_step = c_step+1;

            % for: calculate feedback strengths and perform bootstrapping for positive and
            %      negative feedback strengths
            end

        % for: feedback calculation, month
        end

    % for: feedback calculation, datasets
    end
    textprogressbar('');

    % switch off warning
    warning('on',warn_id);

% if: perform calculations
end

if save_data
% --------- %
% SAVE DATA %
% --------- %

    disp('saving data');

    README = sprintf(['\nVARIABLE LIST\n\n',...
                      'reference_name: Forcing variable\n',...
                      'lag_name:       Response variable\n',...
                      'all_datasets:   Datasets that were compared\n',...
                      'all_data_names: Which dataset is associated with which variable in which cluster?\n',...
                      'all_indices:    Which indices were used for the analysis of each variable?\n',...
                      'analysis_type:  Which analysis type was used to diagnose the feedback?\n',...
                      'start_year:     Start of the analysis period\n',...
                      'stop_year:      End of the analysis period\n\n',...
                      'anomalies:      All anomaly time series used for the analysis\n',...
                      'strengths:      All feedback strengths, for each calendar month\n',...
                      'significances:  Significance of the strength composite, relative to expected strength value']);

    save(out_file,'reference_name','lag_name','all_datasets','all_data_names','all_indices',...
                  'analysis_type','start_year','stop_year',...
                  'anomalies','strengths','significances',...
                  'README');

% -------------------- %
% DISPLAY CORRELATIONS %
% -------------------- %

    disp(sprintf('\nCorrelations:'));

    for anomaly_type = {'positive','negative'};

        anomaly_type = char(anomaly_type);

        use_1 = strengths.(all_datasets{1}).(anomaly_type);
        use_2 = strengths.(all_datasets{2}).(anomaly_type);
        rho = corrcoef(use_1,use_2);

        disp(sprintf('%s: %1.2f',anomaly_type,rho(1,2)));

    end

% load previous data
else

    load(out_file);

% if: save data
end

if plot_results
% ------------ %
% PLOT RESULTS %
% ------------ %

    disp('plotting');

    % SET UP FIGURE
    % +++++++++++++

        % figure
        hf = figure;
        hf.Position(3:4) = [hf.Position(3)*scale_figure_width hf.Position(4)*scale_figure_height];

        % axes
        hax = axes('XLim',[0.5 12.5],'XTick',1:12,'XTickLabels',{month_ids{:}},...
                   'YLim',y_limits.(reference_name),...
                   'Layer','top');
        hax.FontSize = floor(scale_font_size*hax.FontSize);

        hold(hax,'on');
        grid(hax,'on');

        % shade the negative part
        x_values = [hax.XLim(1) hax.XLim(2) hax.XLim(2) hax.XLim(1) hax.XLim(1)];
        y_values = [hax.YLim(1) hax.YLim(1) 0           0           hax.YLim(1)];

        h_fill = patch(x_values,y_values,light_grey);
        set(h_fill,'EdgeColor','none');


        plot(hax.XLim,[0 0],'-','Color',hax.XColor);

        % legend init
        legend_entries = [];
        legend_labels  = {};

    % PLOTTING
    % ++++++++

        x_axis = [1:12];

        for type_string = {'positive','negative'};

            type_string = char(type_string);

            for c_dataset = 1:n_datasets

                dataset_name = char(all_datasets{c_dataset});

                use_data   = strengths.(dataset_name).(type_string);
                use_sigs   = significances.(dataset_name).(type_string);
                use_colour = colours.(type_string);
                use_marker = marker_types.(dataset_name);

                % background
                plot(x_axis,use_data,'-w','LineWidth',4);
                plot(x_axis(use_sigs),use_data(use_sigs),...
                        use_marker,'Color','w','MarkerSize',marker_size+3,'LineWidth',4);

                % colour version
                legend_entries(end+1) = plot(x_axis,use_data,'LineWidth',2,'Color',use_colour);
                plot(x_axis(use_sigs),use_data(use_sigs),...
                        use_marker,'MarkerSize',marker_size,...
                        'MarkerFaceColor',use_colour,'MarkerEdgeColor',use_colour);
                plot(x_axis(~use_sigs),use_data(~use_sigs),...
                        use_marker,'MarkerSize',marker_size,...
                        'MarkerFaceColor',my_grey,'MarkerEdgeColor',my_grey);

            end

            % Correlation values
            corr_series_1 = strengths.(all_datasets{1}).(type_string);
            corr_series_2 = strengths.(all_datasets{2}).(type_string);
            rho = corrcoef(corr_series_1,corr_series_2);

            legend_labels{end+1} = sprintf('%s (%1.2f)',type_string,rho(1,2));

        end

    % LEGEND
    % ++++++

        % fake entries
        legend_entries = legend_entries([1 3]);

        for c_dataset = 1:n_datasets
            dataset_name = char(all_datasets{c_dataset});
            use_marker   = marker_types.(dataset_name);
            legend_entries(end+1) = plot(-1,0,use_marker,...
                    'MarkerSize',marker_size,...
                    'MarkerFaceColor','k','MarkerEdgeColor','k');
            legend_labels{end+1} = dataset_name;
        end

        % create main legend
        h_legend_horizontal = legend(legend_entries(1:2),legend_labels{1:2},...
                'Location','SouthOutside');
        set(h_legend_horizontal,'Orientation','horizontal','Box','off');

        % create secondary axes for second legend
        hax_legend = axes('Position',hax.Position,'FontSize',hax.FontSize,...
                'Color','none','Layer','bottom',...
                'XTickLabels',[],'YTickLabels',[],...
                'Visible','off');
        h_legend_vertical = legend(hax_legend,...
                legend_entries(3:4),legend_labels{3:4},...
                'Location','NorthEastOutside');
        set(h_legend_vertical,'Orientation','vertical','Box','off');

        hold(hax,'off');

    % POLISHING
    % +++++++++

        % naming
        title(hax,fig_title);
        h_ylabel = ylabel(hax,'Feedback strength');
        h_ylabel.Units = 'normalized';
        h_ylabel.Position(1) = ylabel_x_position;

        % align axes
        hax_legend.Position = axes_position;
        hax.Position        = axes_position;
        % hax.Position = hax.Position+axes_offset;

        % printing
        print(hf,res_flag,fig_name,'-dpng');

% if: plot results
end
