%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                            %
%   DECADAL VARIATIONS OF EVENT PARAMETERS   %
%   MAIN                                     %
%                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all;

decadal_variations_settings;

disp(sprintf('\n%s: %s-%s',basin_name,dataset_name,var_name));

if load_data
% --------- %
% LOAD DATA %
% --------- %

    disp('loading ...');

    % load data
    % grids
    grids = load(grid_file);

    % data
    data = load(data_file);

    % time constraint and masking
    % time crop
    is_time = ( data.date_vector(:,1) >= start_year & data.date_vector(:,1) <= stop_year );

    data.monthly_mean = data.monthly_mean(:,:,is_time);
    data.date_vector  = data.date_vector(is_time,:);

    % SORT LONGITUDE DATA
    % Swap 0E away from the matrix edge
    % +++++++++++++++++++++++++++++++++

        if sort_longitude_data

            lon_vector = grids.(dataset_name).lon_grid(:,1);

            if ( strcmp(dataset_name,'ersst') | strcmp(dataset_name,'oras4') )

                disp(['swapping ',dataset_name,' longitudes']);

                % SORT LONGITUDES
                % +++++++++++++++

                    % assumption: Data is ordered ascending or descending
                    % within the positive or negative parts
                    % identify negative parts
                    is_negative = (lon_vector < 0);
                    n_negative  = sum(is_negative);

                    new_lon_vector = zeros(size(lon_vector));

                    % sort longitudes
                    new_lon_vector(1:n_negative) = lon_vector(is_negative);
                    new_lon_vector(n_negative+1:end) = lon_vector(~is_negative);

                % SORT DATA
                % +++++++++

                    old_data = data.monthly_mean;

                    new_data = zeros(size(old_data));
                    new_data(1:n_negative,:,:)     = old_data( is_negative,:,:);
                    new_data(n_negative+1:end,:,:) = old_data(~is_negative,:,:);

                    data.monthly_mean = new_data;

                % SORT GRID DATA
                % ++++++++++++++

                    for swap_name = {'lon_grid','lat_grid','grid_box_area'}

                        swap_name = char(swap_name);
                        disp(sprintf('\t%s',swap_name));

                        old_data = grids.(dataset_name).(swap_name);

                        new_data = zeros(size(old_data));
                        new_data(1:n_negative,:,:)     = old_data( is_negative,:,:);
                        new_data(n_negative+1:end,:,:) = old_data(~is_negative,:,:);

                        grids.(dataset_name).(swap_name) = new_data;

                    end

            % if: swap longitudes for ERSST and ORAS4
            end

        % if: swap around longitude data
        end

% if: load data
end

if perform_calculations
% -------------------- %
% PERFORM CALCULATIONS %
% -------------------- %

    % time axis for chunked analysis
    time_axis = start_year:(stop_year-window_width+1);

    % storage
    strength_data = struct();

    textprogressbar('calculating event characteristics: ');
%      c_lon = 111;
    for c_lon = 1:length(central_longitudes)

        textprogressbar(c_lon/(length(central_longitudes))*100);

        % ANALYSIS REGION
        % +++++++++++++++

            % for this short period: don't detrend the data,
            % just use the local seasonal cycle

            % specify the box
            use_lon = central_longitudes(c_lon);
            use_lat = central_latitudes(c_lon);

            lon_bounds = [(use_lon-lon_extent) (use_lon)+lon_extent];
            lat_bounds = [(use_lat-lat_extent) (use_lat+lat_extent)];

            % index
            index = calcIndexAveraged(data.monthly_mean,...
                    grids.(dataset_name).lat_grid,grids.(dataset_name).lon_grid,...
                    grids.(dataset_name).grid_box_area,...
                    lat_bounds,lon_bounds);

        % loop over all time windows
        for c_start = 1:length(time_axis);

            % correct time
            start_window = time_axis(c_start);
            stop_window  = start_window+window_width-1;

            % anomalies: include linear detrending
            [anomalies,cropped_dates] = calcAnomalies(index,data.date_vector,start_window,stop_window);

            % IDENTIFY EVENTS
            % +++++++++++++++

                for event_type = [1 -1]

                    % naming
                    if ( event_type == 1 )
                        type_string = 'positive';
                    else
                        type_string = 'negative';
                    end

                    if use_anomalies_instead_of_events

                        is_data        = ( sign(anomalies) == event_type );
                        peak_strengths = anomalies(is_data);
                        event_dates    = cropped_dates(is_data,:);

                    % base strength on events
                    else

                        [lengths,growths,decays,event_dates,peak_strengths,mean_strengths] = ...
                                eventLengths(anomalies,cropped_dates,t_id,event_type);

                    end

                    % swap strength data to have ninos and ninas of comparable strengths
                    peak_strengths = abs(peak_strengths);

                    % identify events
                    is_events = ismember(event_dates(:,2),season_def);

                    % RECORD DATA
                    % +++++++++++

                        n_events = sum(is_events);

                        strength_data.(type_string)(c_start,c_lon,1:n_events) = peak_strengths(is_events);

                % for: identify positive and negative events
                end

        % for: identify events for all time chunks
        end

    % for: identify characteristics for all central longitudes
    end
    textprogressbar('');

    % CLEAN UP MISSING VALUES FOR SIGNIFICANCE TESTING
    % ++++++++++++++++++++++++++++++++++++++++++++++++

        for event_type = [-1 1]

            % naming
            if ( event_type == 1 )
                type_string = 'positive';
            else
                type_string = 'negative';
            end

            is_missing = ( strength_data.(type_string) == 0);
            strength_data.(type_string)(is_missing) = NaN;


        % for: clean up missing data
        end

    % SIGNIFICANCE TEST
    % +++++++++++++++++

        disp('testing significance ...');

        is_significant = [];

        % test difference in means for the two parameters
        for c_lon = 1:length(central_longitudes)

            for c_start = 1:length(time_axis);

                use_positive = squeeze(strength_data.positive(c_start,c_lon,:));
                use_negative = squeeze(strength_data.negative(c_start,c_lon,:));

                % exclude NaNs
                use_positive(isnan(use_positive)) = [];
                use_negative(isnan(use_negative)) = [];

                % perform the test
                reject_h0 = ttest2(use_positive,use_negative,...
                        'Alpha',significance_level,'Vartype','unequal');

                % save the decision
                is_significant(c_start,c_lon) = reject_h0;

            % for: significant diffs for both seasons
            end

        % for: signifiat differences between positive and negative events
        end

% if: perform calculations
end

if save_data
% --------- %
% SAVE DATA %
% --------- %

    disp('saving data');

    README = sprintf(['\nVARIABLE LIST\n\n',...
                      'var_name\n',...
                      'dataset_name\n',...
                      'start_year:        Start of the overall analysis period\n',...
                      'stop_year:         End of the overakk analysis period\n',...
                      'window_width:      Over how many years does the running mean average (for each month)?\n',...
                      'central_longitudes\n',...
                      'time_axis:         All start years of the sub-periods analysed via the running mean\n',...
                      'season_def:        Which calendar months have been analysed?\n',...
                      'use_anomalies_instead_of_events: Diagnose anomalie strength or event strength?\n',...
                      't_id:              Identification threshold in terms of the local variance\n',...
                      'significance_level\n\n',...
                      'strength_data:     For positive and negative events\n',...
                      'is_significant:    Corresponding significance']);

    save(out_file,...
         'var_name','dataset_name','start_year','stop_year','window_width',...
         'central_longitudes','time_axis','season_def','use_anomalies_instead_of_events',...
         't_id','significance_level',...
         'strength_data','is_significant',...
         'README');

end

if load_previous_data
    load(out_file);
end

if produce_plots
% ----- %
% PLOTS %
% ----- %

    decadal_variations_plots;

end
