%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                 %
%    STRENGTH OF LAGGED BJERKNES FEEDBACK ELEMENTS                %
%    VARIABLE ANALYSIS TYPE: ACC, REGRESSION, ROBUST REGRESSION   %
%    MAIN                                                         %
%                                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

lagged_feedback_strengths_settings;

disp(sprintf('\n\nSEASONAL LAGGED CORRELATIONS VIA %s: %s %s-%s, %d-%d\n',...
        upper(analysis_type),basin_name,upper(all_vars{1}),upper(all_vars{2}),...
        start_year,stop_year));

if load_data
% --------- %
% LOAD DATA %
% --------- %

    disp('loading data');
    data = load(infile);

% if: load data
end

if perform_calculations
% -------------------- %
% PERFORM CALCULATIONS %
% -------------------- %

    % switch rreg warning back on
    if ( analysis_id == 3 )
        warning('off',warn_id);
    end

    % storage
    feedback_strengths = struct();
    significance = struct();

    % ignore data where the lag relationship between the two variables is negative
    % (i.e. the lagged variable leads the response; this is in disagreement
    % with the Bjerknes feedback framework -- hence I include these cases).
    is_negative_lag = ( data.lag_values < 0 )';  % need to transpose here for dimensions

    % loop over all boxes
    textprogressbar(['calculating seasonal feedback strengths via ',analysis_title,': '])
    for c_lon = 1:length(central_longitudes)

        textprogressbar(c_lon/(length(central_longitudes))*100);

        for month = 1:12

            % select data
            use_data = struct();

            for var_name = {all_vars{:}}

                var_name  = char(var_name);
                use_data.(var_name) = squeeze(data.indices.(var_name)(c_lon,month,:));

                % remove NaNs
                is_nan = isnan(use_data.(var_name));
                use_data.(var_name)(is_nan) = [];

            end

            % quality check
            n_var1 = length(use_data.(all_vars{1}));
            n_var2 = length(use_data.(all_vars{2}));

            if n_var1 ~= n_var2
                error(['Something is wrong with the time dimension of the data.\n',...
                        'Monthly mean of ',upper(all_vars{1}),' and ',upper(all_vars{2}),...
                        ' must span the same period.']);
            end

            % calculate feedback_strengths for all anomaly types
            for anomaly_type = all_anomalies

                % assign correct anomaly type
                switch anomaly_type
                    case -1
                        type_string = 'negative';
                    case 0
                        type_string = 'full';
                    case 1
                        type_string = 'positive';
                end

                % choose correct data
                if ( anomaly_type == 0 )
                    is_data = 1:n_var1;
                else
                    is_data = ( sign(use_data.(all_vars{1})) == anomaly_type );
                end

                % THE ACTUAL FEEDBACK STRENGTH
                % ++++++++++++++++++++++++++++

                    switch analysis_id

                        % correlation
                        case 1

                            % acc
                            current_feedback = corrcoef(use_data.(all_vars{1})(is_data),...
                                use_data.(all_vars{2})(is_data));
                            current_feedback = current_feedback(1,2);

                        % normal linear regression
                        case 2

                            regression_parameters = polyfit(use_data.(all_vars{1})(is_data),...
                                    use_data.(all_vars{2})(is_data),1);
                            current_feedback = regression_parameters(1);

                        % robust regression
                        case 3

                            rregression_parameters = robustfit(use_data.(all_vars{1})(is_data),...
                                    use_data.(all_vars{2})(is_data));
                            current_feedback = rregression_parameters(2);  % yes, the order is different from polyfit ...

                    % switch: calculate feedback strength
                    end

                    % assign correlation coefficients
                    feedback_strengths.(type_string)(month,c_lon) = current_feedback;

                % THE BOOTSTRAPPED SIGNIFICANCE
                % +++++++++++++++++++++++++++++

                    % number for bootstrapping approach
                    n_data = sum(is_data);

                    % bootstrap distribution for significance test
                    bootstrapped_strengths = bootstrapFeedbackStrength(...
                            use_data.(all_vars{1}),use_data.(all_vars{2}),...
                            n_data,n_bootstraps,analysis_type);

                    % perform actual significance test
                    current_significance = testValue(bootstrapped_strengths,current_feedback,alpha);

                    % assign significance
                    significance.(type_string)(month,c_lon) = current_significance;

            % for: perform feedback analysis for both positive and negative anomalies
            end

        % for: perform correlation analysis for all months
        end

    % for: calculate feedback strengths for all longitudes
    end
    textprogressbar('');

    % switch rreg warning back on
    if ( analysis_id == 3 )
        warning('on',warn_id);
    end

% if: perform calculations
end

if save_data
% ------------------------------------------- %
% SAVE DATA / LOAD PREVIOUSLY CALCULATED DATA %
% ------------------------------------------- %

    disp('saving data');

    README = ['\nVARIABLE LIST\n\n',...
              'basin_name\n',...
              'reference_name:     forcing variable in the Bjerknes feedback framework\n',...
              'lag_name:           Variable that was moved around the reference to diagnose the lag\n',...
              'all_data:           Datasets for each variable\n',...
              'start_year\n',...
              'stop_year\n',...
              'analysis_type:      String describing the analysis method:\n',...
              '                    acc: Anomaly correlation coefficients\n',...
              '                    reg: linear regression\n',...
              '                    rreg: robust regression (robustfit)\n',...
              'analysis_id:        ID of the analysis method\n\n',...
              'central_longitudes\n',...
              'alpha:              Significance level for the bootstrapping\n\n',...
              'feedback_strengths: Strength of the feedback\n',...
              'is_negative_lag:    Boolean that is 0 when the lag at this pixel is negative\n',...
              'significance:       For pos/neg strengths: Significantly different from general strength?\n',...
              '                    1: significant at alpha level, 0: insignificant\n\n',...
              'CORRESPONDING DATA FILE, includes the lags:\n',...
              infile];

    save(outfile,...
         'basin_name','reference_name','lag_name','all_data',...
         'start_year','stop_year',...
         'analysis_type','analysis_id',...
         'central_longitudes','alpha',...
         'feedback_strengths','is_negative_lag','significance',...
         'README');

end

% ----- %
% PLOTS %
% ----- %

    % omit calculation and use previous data
    if load_previous_data
        load(outfile);
    end

    % plot feedback strengths of negative and positive composites
    if produce_plots
        lagged_feedback_strengths_plot_strength_composites;
    end
