%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                           %
%   CALCULATE TOTAL BJERKNES FEEDBACK BASED ON REGRESSION   %
%   FOR ZONAL AND SEASONAL CALCULATIONS                     %
%   MAIN                                                    %
%                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
clc;
close all;

total_bjerknes_feedback_from_regression_settings;
disp(sprintf('%s: %s',basin_name,dataset_id));

% --------- %
% LOAD DATA %
% --------- %

    disp('loading data');

    n_feedbacks = length(all_references);
    data        = struct();

    for c_feedback = 1:n_feedbacks

        ref_name = char(all_references{c_feedback});
        lag_name = char(all_lags{c_feedback});

        % infile
        in_file = strrep(file_dummy,'@REF_DATA@',all_data.(ref_name));
        in_file = strrep(in_file,'@REF_NAME@',ref_name);
        in_file = strrep(in_file,'@LAG_DATA@',all_data.(lag_name));
        in_file = strrep(in_file,'@LAG_NAME@',lag_name);

        data.(ref_name) = load(in_file);

    % for: load all feedback data
    end

% -------------- %
% NORMALIZE DATA %
% -------------- %

    % preparation
    dummy = zeros(size(data.(ref_name).feedback_strengths.positive));

    % storage
    normal_values = struct();
    normalized    = struct();

    % where is the feedback loop broken, i.e.
    % where is at least one of the feedback elements negative?
    broken_loop   = struct('positive',logical(dummy),'negative',logical(dummy));

    % recursively add up data
    total_feedback = broken_loop;

    % size of the data
    n_elements = numel(data.ssh.feedback_strengths.positive);

    c_feedback = 1;
    for c_feedback = 1:n_feedbacks

        ref_name = char(all_references{c_feedback});

        % find the normalizing value
        % put all relevant data into one vector
        data_vector = [reshape(data.(ref_name).feedback_strengths.negative,[1,n_elements]),...
                       reshape(data.(ref_name).feedback_strengths.positive,[1,n_elements])];

        % sort data and find the value that corresponds to the chosen level
        sorted_data = sort(data_vector);

        % "integrate" the sorted data, normalize the integration
        estimated_cdf = cumsum(sorted_data)./sum(sorted_data);

        % find the chosen level and the corresponding feedback strength
        [minimum_distance cdf_index] = min(abs(estimated_cdf-normalizing_level));

        % corresponding strength value
        % use contant value for direct comparison
        % between basins? or the value found here?
        if use_constant_normalization
            normal_values.(ref_name) = cut_off_levels.(ref_name);
        else
            normal_values.(ref_name) = sorted_data(cdf_index);
        end

        % actual normlizaton of the data
        % ignore everything that is negative
        for anomaly_type = {'negative','positive'}

            anomaly_type = char(anomaly_type);

            use_data = data.(ref_name).feedback_strengths.(anomaly_type);

            % actual normalization
            use_data = use_data./normal_values.(ref_name);

            % set everything to 1 that is higher than 0
            % (should be 5% of the total data)
            use_data(use_data > 1) = 1;
            use_data(use_data < -1) = -1;

            % save data
            normalized.(ref_name).(anomaly_type) = use_data;

            % loop?
            broken_loop.(anomaly_type) = ...
                     ( broken_loop.(anomaly_type) | ( use_data < 0 ) );

            % recursively multiply up all data
            total_feedback.(anomaly_type) = total_feedback.(anomaly_type)+use_data;

        % normalize positive and negative data
        end

    % for: normalize feedback data
    end

% for testing
total_feedback.negative = total_feedback.negative./n_feedbacks;
total_feedback.positive = total_feedback.positive./n_feedbacks;

if plot_total_strengths
% ------------------- %
% PLOT TOTAL FEEDBACK %
% ------------------- %

    disp('plotting');

    for anomaly_type = {'negative','positive'}

        anomaly_type = char(anomaly_type);
        disp(sprintf('\t%s',anomaly_type));

        % get data
        use_data = total_feedback.(anomaly_type);

        % set NaNs for broken loops
        use_loop = broken_loop.(anomaly_type);
        use_data(use_loop) = NaN;

        % PREPARE FIGURE
        % ++++++++++++++

            hf  = figure;
            hf.Position(3) = hf.Position(3)*scale_figure_width;
            hf.Position(4) = hf.Position(4)*scale_figure_height;

            % initialize axes
            hax = axes('XLim',lon_cover,'YLim',[0.5 12.5],...
                    'YTick',[1:12],'YTickLabel',month_ids,...
                    'XGrid','on','Layer','top');
            hax.FontSize = scale_font_size*hax.FontSize;

            hold(hax,'on');

        % PLOT MAIN DATA
        % ++++++++++++++

            % checkerboard, as in the dynamics analysis
            % I definitely can't use pcolor here -- in fact, I should never
            % use it -- because it is averaging the adjacent values together
            % to produce the cell value. Hence, I'm effectively losing data,
            % and this is very much not what I want to do.
            image(lon_cover,[1 12],use_data,...
                    'CDataMapping','scaled');
            hcb = colorbar(hax);

        % NINO REGIONS
        % ++++++++++++

            % label boxes in a different axis
            hax_labels = axes('XLim',lon_cover,'YLim',[0.5 12.5],...
                    'Position',hax.Position,'FontSize',hax.FontSize,...
                    'Visible','off');

            hold(hax_labels,'on');

            for c_region = 1:length(region_names)

                x_values = [highlight_regions(c_region,1) highlight_regions(c_region,2) ...
                            highlight_regions(c_region,2) highlight_regions(c_region,1) ...
                            highlight_regions(c_region,1)];

                y_values = [highlight_y_values(c_region,1) highlight_y_values(c_region,1) ...
                            highlight_y_values(c_region,2) highlight_y_values(c_region,2) ...
                            highlight_y_values(c_region,1)];

                h_patch = patch(x_values,y_values,region_colours(c_region,:),...
                        'EdgeColor','none');

            % plot in highlight regions
            end

        % FINISH PLOT AND PRINT
        % +++++++++++++++++++++

            % polishing
            xlabel(hax,'\circ lon');
            ylabel(hax,'Month');

            fig_title = strrep(total_title_dummy,'@ANOMALYTYPE@',anomaly_type);
            h_title = title(hax_labels,fig_title);
            set(h_title,'Visible','on');

            % align axes
            hax_labels.Position = axes_position;
            hax.Position        = axes_position;

            % offset the patches
            hax_labels.Position(2)   = hax_labels.Position(2)+hax_labels_height_offset.(basin_name);

            % colormap and number of displayed colours
            hax.CLim = c_limits;
            use_cmap = plasma(n_colours);

            colormap(use_cmap);

            % print
            fig_name = strrep(total_name_dummy,'@ANOMALYTYPE@',anomaly_type);
            print(hf,print_res,fig_name,'-dpng');
            close(hf);

    % for: plot all total feedbacks
    end

% if: plot total strengths
end

if plot_total_strength_difference
% ------------------------- %
% TOTAL STRENGTH DIFFERENCE %
% ------------------------- %

    disp('plotting strength difference');

    % get data
    use_data = total_feedback.positive-total_feedback.negative;

    % NaN for broken loops
    use_loop           = ( broken_loop.positive | broken_loop.negative );
    use_data(use_loop) = 0;

    % PREPARE FIGURE
    % ++++++++++++++

        hf  = figure;
        hf.Position(3) = hf.Position(3)*scale_figure_width;
        hf.Position(4) = hf.Position(4)*scale_figure_height;

        % initialize axes
        hax = axes('XLim',lon_cover,'YLim',[0.5 12.5],...
                'YTick',[1:12],'YTickLabel',month_ids,...
                'XGrid','on','Layer','top');
        hax.FontSize = scale_font_size*hax.FontSize;

        hold(hax,'on');

    % PLOT MAIN DATA
    % ++++++++++++++

        % checkerboard, as in the dynamics analysis
        % I definitely can't use pcolor here -- in fact, I should never
        % use it -- because it is averaging the adjacent values together
        % to produce the cell value. Hence, I'm effectively losing data,
        % and this is very much not what I want to do.
        image(lon_cover,[1 12],use_data,...
                'CDataMapping','scaled');
        hcb = colorbar(hax);

    % NINO REGIONS
    % ++++++++++++

        % label boxes in a different axis
        hax_labels = axes('XLim',lon_cover,'YLim',[0.5 12.5],...
                'Position',hax.Position,'FontSize',hax.FontSize,...
                'Visible','off');

        hold(hax_labels,'on');

        for c_region = 1:length(region_names)

            x_values = [highlight_regions(c_region,1) highlight_regions(c_region,2) ...
                        highlight_regions(c_region,2) highlight_regions(c_region,1) ...
                        highlight_regions(c_region,1)];

            y_values = [highlight_y_values(c_region,1) highlight_y_values(c_region,1) ...
                        highlight_y_values(c_region,2) highlight_y_values(c_region,2) ...
                        highlight_y_values(c_region,1)];

            h_patch = patch(x_values,y_values,region_colours(c_region,:),...
                    'EdgeColor','none');

        % plot in highlight regions
        end

    % FINISH PLOT AND PRINT
    % +++++++++++++++++++++

        % polishing
        xlabel(hax,'\circ lon');
        ylabel(hax,'Month');

        fig_title = difference_title_dummy;
        h_title = title(hax_labels,fig_title);
        set(h_title,'Visible','on');

        % align axes
        hax_labels.Position = axes_position;
        hax.Position        = axes_position;

        % offset the patches
        hax_labels.Position(2) = hax_labels.Position(2)+hax_labels_height_offset.(basin_name);

        % colormap and number of displayed colours
        hax.CLim = [-0.8 0.8];     % for comparison with total feedbacks based on a single compositing variable
        use_cmap = centered(25);   % for comparison with total feedbacks based on a single compositing variable

        colormap(use_cmap);

        % print
        fig_name = difference_name_dummy;
        print(hf,print_res,fig_name,'-dpng');
        close(hf);

end

%  if plot_normalized_feedbacks
%  % --------------- %
%  % NORMALIZED DATA %
%  % --------------- %
%
%      % plot all normalized data
%      for c_feedback = 1:n_feedbacks
%
%          ref_name = char(all_references{c_feedback});
%          lag_name = char(all_lags{c_feedback});
%
%          disp([ref_name,'-',lag_name]);
%
%          for anomaly_type = {'negative','positive'}
%
%              anomaly_type = char(anomaly_type);
%              disp(sprintf('\t%s',anomaly_type));
%
%              % get data
%              use_data = normalized.(ref_name).(anomaly_type);
%
%              % PREPARE FIGURE
%              % ++++++++++++++
%
%                  hf  = figure;
%                  hf.Position(3) = hf.Position(3)*scale_figure_width;
%                  hf.Position(4) = hf.Position(4)*scale_figure_height;
%
%                  % initialize axes
%                  hax = axes('XLim',lon_cover,'YLim',[0.5 12.5],...
%                          'YTick',[],'YTickLabel','',...
%                          'XGrid','on','Layer','top');
%                  hax.FontSize = scale_font_size*hax.FontSize;
%
%                  hold(hax,'on');
%
%                  % naming the regions
%                  x_ticks        = [];
%                  x_tick_labels  = {};
%
%              % PLOT MAIN DATA
%              % ++++++++++++++
%
%                  % checkerboard, as in the dynamics analysis
%                  % I definitely can't use pcolor here -- in fact, I should never
%                  % use it -- because it is averaging the adjacent values together
%                  % to produce the cell value. Hence, I'm effectively losing data,
%                  % and this is very much not what I want to do.
%                  image(lon_cover,[1 12],use_data,...
%                          'CDataMapping','scaled');
%                  hcb = colorbar(hax);
%
%              % NINO REGIONS
%              % ++++++++++++
%
%                  % label boxes in a different axis
%                  hax_labels = axes('XLim',lon_cover,'YLim',[1 13],...
%                          'YTick',[1.5:1:12.5],'YTickLabel',month_ids);
%                  set(hax_labels,'Color','none','XAxisLocation','top',...
%                          'Position',hax.Position,'FontSize',hax.FontSize);
%
%                  % Nino regions
%                  for c_region = 1:length(region_names)
%
%                      x_values = [highlight_regions(c_region,1) highlight_regions(c_region,2) ...
%                                  highlight_regions(c_region,2) highlight_regions(c_region,1) ...
%                                  highlight_regions(c_region,1)];
%                      y_values = [hax.YLim(1) hax.YLim(1) hax.YLim(2) hax.YLim(2) hax.YLim(1)];
%
%                      plot(x_values,y_values,'LineWidth',4,'Color',my_white);
%                      plot(x_values,y_values,'LineWidth',2,'Color',region_colours(c_region,:));
%
%                      x_ticks(end+1)       = mean(highlight_regions(c_region,:));
%                      x_tick_labels{end+1} = region_names{c_region};
%
%                  % plot in highlight regions
%                  end
%
%              % FINISH PLOT AND PRINT
%              % +++++++++++++++++++++
%
%                  % polishing
%                  xlabel(hax,'\circ lon');
%                  ylabel(hax_labels,'Month');
%
%                  fig_title = strrep(feedback_title_dummy,'@ANOMALYTYPE@',anomaly_type);
%                  fig_title = strrep(fig_title,'@REF_NAME@',upper(ref_name));
%                  fig_title = strrep(fig_title,'@LAG_NAME@',upper(lag_name));
%                  title(hax_labels,fig_title);
%
%                  % align axes
%                  hax_labels.Position = axes_position;
%                  hax.Position        = axes_position;
%
%                  % colormap and number of displayed colours
%                  hax.CLim = [-1 1];
%                  use_cmap = centered(19);
%
%                  % ignore first entry, since it also covers the NaNs
%                  colormap(use_cmap);
%
%                  % display region names
%                  set(hax_labels,'XTick',x_ticks,'XTickLabel',x_tick_labels);
%
%                  % print
%                  fig_name = strrep(feedback_name_dummy,'@ANOMALYTYPE@',anomaly_type);
%                  fig_name = strrep(fig_name,'@REF_NAME@',ref_name);
%                  fig_name = strrep(fig_name,'@LAG_NAME@',lag_name);
%                  print(hf,print_res,fig_name,'-dpng');
%                  close(hf);
%
%              % for: plot all normalized data, anomaly type
%              end
%
%      % for: plot all normalized data, feedback
%      end
%
%  % if: plot normalized feedbacks
%  end
